//plugincontainer.h:

/*
 *      Copyright (C) Philipp 'ph3-der-loewe' Schafft - 2008-2012
 *
 *  This file is part of libroar a part of RoarAudio,
 *  a cross-platform sound system for both, home and professional use.
 *  See README for details.
 *
 *  This file is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3
 *  as published by the Free Software Foundation.
 *
 *  libroar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 *  NOTE for everyone want's to change something and send patches:
 *  read README and HACKING! There a addition information on
 *  the license of this document you need to read before you send
 *  any patches.
 *
 *  NOTE for uses of non-GPL (LGPL,...) software using libesd, libartsc
 *  or libpulse*:
 *  The libs libroaresd, libroararts and libroarpulse link this lib
 *  and are therefore GPL. Because of this it may be illigal to use
 *  them with any software that uses libesd, libartsc or libpulse*.
 */

#ifndef _LIBROARPLUGINCONTAINER_H_
#define _LIBROARPLUGINCONTAINER_H_

#include "libroar.h"

struct roar_plugincontainer;

/* Create a new plugin container.
 * Takes a default parameter set.
 */
struct roar_plugincontainer * roar_plugincontainer_new(struct roar_dl_librarypara * default_para);

/* Create a new plugin container.
 * Takes host application's appname and abiversion.
 */
struct roar_plugincontainer * roar_plugincontainer_new_simple(const char * appname, const char * abiversion);

// Increment the refrence counter.
int roar_plugincontainer_ref(struct roar_plugincontainer * cont);
/* Decrement the refrence counter.
 * Unloads all plugins and frees all resources when there are no refreneces left.
 */
int roar_plugincontainer_unref(struct roar_plugincontainer * cont);

/*
 * stuff we still need:
 * object creation, get plugin handle.
 */

// plugin loading and unloading:

// Load a plugin by name.
int                      roar_plugincontainer_load            (struct roar_plugincontainer * cont,
                                                               const char * name,
                                                               struct roar_dl_librarypara * para);

/* Load a plugin by name with extra options.
 * This is for advanced applications only.
 * NOTE: Using this handle after the plugin has been unloaded from the
 *       container results in undefind behavior.
 */
struct roar_dl_lhandle * roar_plugincontainer_load_lhandle    (struct roar_plugincontainer * cont,
                                                               const char * name,
                                                               int flags,
                                                               int ra_init,
                                                               struct roar_dl_librarypara * para);
/* Unload a plugin by name.
 * NOTE: The name here is from the plugin and may not match the name you load
 *       the plugin with.
 */
int                      roar_plugincontainer_unload          (struct roar_plugincontainer * cont,
                                                               const char * name);
/* Load a plugin by roardl handle.
 * This is for advanced applications only.
 */
int                      roar_plugincontainer_unload_lhandle  (struct roar_plugincontainer * cont,
                                                               struct roar_dl_lhandle * lhandle);
/* Post ra_init plugins not yet inited.
 * NOTE: This uses the default para,
 *       not the one given with roar_plugincontainer_load_lhandle().
 * This is for advanced applications only.
 */
int                      roar_plugincontainer_ra_init         (struct roar_plugincontainer * cont);

// appsched:
// Trigger an application schedule event on all plugins.
int                      roar_plugincontainer_appsched_trigger(struct roar_plugincontainer * cont, enum roar_dl_appsched_trigger trigger);

#endif

//ll
